==> GotoBLAS-1.10.tar.gz: GotoBLAS/level3/syr2k/zsyr2k.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

#ifdef XDOUBLE
#define ERROR_NAME "XSYR2K"
#elif defined(DOUBLE)
#define ERROR_NAME "ZSYR2K"
#else
#define ERROR_NAME "CSYR2K"
#endif

static int (*syrk_beta[])(long, long, long, FLOAT, FLOAT,
			  FLOAT *, long, FLOAT* , long, FLOAT *, long, FLOAT *, long, long) = {
#ifdef XDOUBLE
	       xsyrk_beta_U, xsyrk_beta_L,
#elif defined(DOUBLE)
	       zsyrk_beta_U, zsyrk_beta_L,
#else
	       csyrk_beta_U, csyrk_beta_L,
#endif
    };

static int (*syr2k[])(long, long, long, FLOAT, FLOAT,
		      FLOAT *, long, FLOAT *, long, FLOAT *, long, FLOAT *, long, long) = {
#ifdef XDOUBLE
    xsyr2k_UN, xsyr2k_UT, xsyr2k_LN, xsyr2k_LT,
#elif defined(DOUBLE)
    zsyr2k_UN, zsyr2k_UT, zsyr2k_LN, zsyr2k_LT,
#else
    csyr2k_UN, csyr2k_UT, csyr2k_LN, csyr2k_LT,
#endif
  };

int NAME(char *UPLO, char *TRANS,
         blasint *N, blasint *K, 
         FLOAT *ALPHA, FLOAT *a, blasint *LDA, 
	               FLOAT *b, blasint *LDB,
         FLOAT *BETA,  FLOAT *c, blasint *LDC){
  
  char uplo_arg  = *UPLO;
  char trans_arg = *TRANS;

  blasint n = *N;
  blasint k = *K;
  FLOAT alpha_r = *(ALPHA + 0);
  FLOAT alpha_i = *(ALPHA + 1);
  FLOAT beta_r  = *(BETA  + 0);
  FLOAT beta_i  = *(BETA  + 1);
  blasint lda = *LDA;
  blasint ldb = *LDB;
  blasint ldc = *LDC;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

  blasint info;
  int uplo;
  int trans;
  int nrowa;

  TOUPPER(uplo_arg);
  TOUPPER(trans_arg);

  uplo  = -1;
  trans = -1;

  if (uplo_arg  == 'U') uplo  = 0;
  if (uplo_arg  == 'L') uplo  = 1;

  if (trans_arg == 'N') trans = 0;
  if (trans_arg == 'T') trans = 1;
  
  nrowa = n;
  if (trans & 1) nrowa = k;

  info = 0;

  if (ldc < MAX(1,n))     info = 12;
  if (ldb < MAX(1,nrowa)) info =  9;
  if (lda < MAX(1,nrowa)) info =  7;
  if (k < 0)              info =  4;
  if (n < 0)              info =  3;
  if (trans < 0)          info =  2;
  if (uplo  < 0)          info =  1;

  if (info != 0) {
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return 0;
  }
  
  if (n == 0) return 0;
  
#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP

  if ((beta_r != ONE) || (beta_i != ZERO)){
    (syrk_beta[uplo])(0, n, 0, beta_r, beta_i, NULL, 0, NULL, 0, c, ldc, NULL, 0, n);
  }

#else

#ifdef DOUBLE
  mode  =  BLAS_DOUBLE | BLAS_COMPLEX;
#else
  mode  =  BLAS_SINGLE | BLAS_COMPLEX;
#endif

  if (!trans){
    mode |= (BLAS_TRANSA_N | BLAS_TRANSB_T);
  } else {
    mode |= (BLAS_TRANSA_T | BLAS_TRANSB_N);
  }

  mode |= (uplo  << BLAS_UPLO_SHIFT);
 
  if ((beta_r != ONE) || (beta_i != ZERO))
    syrk_thread(mode, n, k, beta_r, beta_i,
		a, lda, NULL, 0, c, ldc, 
		(void *)syrk_beta[uplo], NULL);
#endif

  if ((alpha_r == 0.) && (alpha_i == 0.)) return 0;

  if (k == 0) return 0;

  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

  (syr2k[(uplo << 1) | trans ])
    (0, n, k, alpha_r, alpha_i, a, lda, b, ldb, c, ldc, buffer, 0, n);

#else

 syrk_thread(mode, n, k, alpha_r, alpha_i,
	     a, lda, b, ldb, c, ldc, 
	      (void *)syr2k[(uplo<<1) | trans], buffer);

#endif

  blas_memory_free(buffer);

  return 0;
}

==> GotoBLAS-1.12.tar.gz: GotoBLAS/level3/syr2k/zsyr2k.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

#ifdef XDOUBLE
#define ERROR_NAME "XSYR2K"
#elif defined(DOUBLE)
#define ERROR_NAME "ZSYR2K"
#else
#define ERROR_NAME "CSYR2K"
#endif

static int (*syrk_beta[])(long, long, long, FLOAT, FLOAT,
			  FLOAT *, long, FLOAT* , long, FLOAT *, long, FLOAT *, long, long) = {
#ifdef XDOUBLE
	       xsyrk_beta_U, xsyrk_beta_L,
#elif defined(DOUBLE)
	       zsyrk_beta_U, zsyrk_beta_L,
#else
	       csyrk_beta_U, csyrk_beta_L,
#endif
    };

static int (*syr2k[])(long, long, long, FLOAT, FLOAT,
		      FLOAT *, long, FLOAT *, long, FLOAT *, long, FLOAT *, long, long) = {
#ifdef XDOUBLE
    xsyr2k_UN, xsyr2k_UT, xsyr2k_LN, xsyr2k_LT,
#elif defined(DOUBLE)
    zsyr2k_UN, zsyr2k_UT, zsyr2k_LN, zsyr2k_LT,
#else
    csyr2k_UN, csyr2k_UT, csyr2k_LN, csyr2k_LT,
#endif
  };

int NAME(char *UPLO, char *TRANS,
         blasint *N, blasint *K, 
         FLOAT *ALPHA, FLOAT *a, blasint *LDA, 
	               FLOAT *b, blasint *LDB,
         FLOAT *BETA,  FLOAT *c, blasint *LDC){
  
  char uplo_arg  = *UPLO;
  char trans_arg = *TRANS;

  blasint n = *N;
  blasint k = *K;
  FLOAT alpha_r = *(ALPHA + 0);
  FLOAT alpha_i = *(ALPHA + 1);
  FLOAT beta_r  = *(BETA  + 0);
  FLOAT beta_i  = *(BETA  + 1);
  blasint lda = *LDA;
  blasint ldb = *LDB;
  blasint ldc = *LDC;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

  blasint info;
  int uplo;
  int trans;
  int nrowa;

  TOUPPER(uplo_arg);
  TOUPPER(trans_arg);

  uplo  = -1;
  trans = -1;

  if (uplo_arg  == 'U') uplo  = 0;
  if (uplo_arg  == 'L') uplo  = 1;

  if (trans_arg == 'N') trans = 0;
  if (trans_arg == 'T') trans = 1;
  
  nrowa = n;
  if (trans & 1) nrowa = k;

  info = 0;

  if (ldc < MAX(1,n))     info = 12;
  if (ldb < MAX(1,nrowa)) info =  9;
  if (lda < MAX(1,nrowa)) info =  7;
  if (k < 0)              info =  4;
  if (n < 0)              info =  3;
  if (trans < 0)          info =  2;
  if (uplo  < 0)          info =  1;

  if (info != 0) {
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return 0;
  }
  
  if (n == 0) return 0;
  
#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP

  if ((beta_r != ONE) || (beta_i != ZERO)){
    (syrk_beta[uplo])(0, n, 0, beta_r, beta_i, NULL, 0, NULL, 0, c, ldc, NULL, 0, n);
  }

#else

#ifdef XDOUBLE
  mode  =  BLAS_XDOUBLE | BLAS_COMPLEX;
#elif defined(DOUBLE)
  mode  =  BLAS_DOUBLE  | BLAS_COMPLEX;
#else
  mode  =  BLAS_SINGLE  | BLAS_COMPLEX;
#endif  

  if (!trans){
    mode |= (BLAS_TRANSA_N | BLAS_TRANSB_T);
  } else {
    mode |= (BLAS_TRANSA_T | BLAS_TRANSB_N);
  }

  mode |= (uplo  << BLAS_UPLO_SHIFT);
 
  if ((beta_r != ONE) || (beta_i != ZERO))
    syrk_thread(mode, n, k, beta_r, beta_i,
		a, lda, NULL, 0, c, ldc, 
		(void *)syrk_beta[uplo], NULL);
#endif

  if ((alpha_r == 0.) && (alpha_i == 0.)) return 0;

  if (k == 0) return 0;

  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

  (syr2k[(uplo << 1) | trans ])
    (0, n, k, alpha_r, alpha_i, a, lda, b, ldb, c, ldc, buffer, 0, n);

#else

 syrk_thread(mode, n, k, alpha_r, alpha_i,
	     a, lda, b, ldb, c, ldc, 
	      (void *)syr2k[(uplo<<1) | trans], buffer);

#endif

  blas_memory_free(buffer);

  return 0;
}
